package primerdesigner;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;
import org.synbiowave.AbstractSynBioWaveRobot;
import org.synbiowave.biojava.SequenceUtils;
import org.synbiowave.menu.MenuCallBack;
import org.synbiowave.menu.MenuItem;
import org.synbiowave.menu.items.Formular;
import org.synbiowave.workspace.RichSequence;
import org.synbiowave.workspace.Sequence;
import org.synbiowave.workspace.Workspace;

import com.google.wave.api.Blip;
import com.google.wave.api.Gadget;
import com.google.wave.api.event.Event;

public class PrimerDesignCallback extends MenuCallBack {
	
	int temp = 60;
	String primerPrefix = new String("");
	String primerPrefix2 = new String("");
	private int start = 0;
	private int end = 0;
	
	final int MIN_PRIMERLENGTH = 10;
	private final String publishGadgetURL = "http://synbiowave-primerdesigner.appspot.com/primerGadget.xml";
	
	public void onFormSubmit(Event event, Formular menuItem, AbstractSynBioWaveRobot servlet)
	{
		// Load the sequences from workspace
		List<Sequence> sequences =  Workspace.getActiveSequences(servlet, event.getWavelet());
		
		if(sequences.size() != 1)
		{
			event.getBlip().reply().append("Please selecte one (and only one) Sequence from the Workspace!");
			return;
		}
		
		EvaluateForm(menuItem, sequences.get(0).getSequence().length());
		
		// Parse the sequence as RichSequence
		RichSequence target = new RichSequence(sequences.get(0));
		
		if(!target.isDNA()) 
		{
			event.getBlip().reply().append("The selected Sequence is no DNA. Please select a DNA Sequence!");
			return;
		}
		
		// generate the Primer
		RichSequence primer1 = new RichSequence();
		RichSequence primer2 = new RichSequence();
		
		try{
			primer1 = GeneratePrimer(target, temp, start);
			primer1.setPrefix(primerPrefix);
			primer2 = GeneratePrimer(target.ReverseComplement(), 
					primer1.getMeltingPoint(true), end);
			primer2.setPrefix(primerPrefix2);
			}catch(Exception e)
		{
			event.getBlip().reply().append("Sorry, apparently your target sequence is to short to design a primer!"
					+ e.getStackTrace());
			return;
		}
			
		ArrayList<Sequence> display = new ArrayList<Sequence>();
		display.add(new Sequence("Primer 1:",
				RichSequence.generateLeadingGaps(start)+
				primer1.getPrefix()+primer1.getSequence()
				+RichSequence.generateLeadingGaps(
						target.getSequence().length() -
						start -
						primer1.getSequence().length()
						+ primer2.getPrefix().length())));
		display.add(new Sequence("Seq:",
				RichSequence.generateLeadingGaps(primer1.getPrefix().length())
				+ target.getSequence() 
				+ RichSequence.generateLeadingGaps(primer2.getPrefix().length())));
		display.add(new Sequence("RevComp:",
				RichSequence.generateLeadingGaps(primer1.getPrefix().length())
				+ target.getComplement().getSequence()
				+ RichSequence.generateLeadingGaps(primer2.getPrefix().length())));
		display.add(new Sequence("Primer 2:",
				RichSequence.generateLeadingGaps(primer1.getPrefix().length() +
						target.getSequence().length() - primer2.getSequence().length() - end)
						+ new StringBuffer(primer2.getSequence()).reverse().toString()
						+ new StringBuffer(primer2.getPrefix()).reverse().toString()
						+ RichSequence.generateLeadingGaps(end)));
		
		DecimalFormat twoPlaces = new DecimalFormat("0.00");
	
		
		Blip output = event.getWavelet().reply(
				"\nPrimer 1 melting temp: " + twoPlaces.format(primer1.getMeltingPoint(true)) + "°C" +
				"\nLow Primer 2 melting temp: " + twoPlaces.format(primer2.getMeltingPoint(true)) + "°C" +
				"\n");
		
		SequenceUtils.showGadgetView(output, display);
		
		// Publishing
		primer1.setAuthor("Synbiowave Primerdesigner");
		primer2.setAuthor("Synbiowave Primerdesigner");
		primer1.setType(SequenceUtils.determineType(primer1.getSequence()));
		primer2.setType(SequenceUtils.determineType(primer2.getSequence()));
		
		Gadget button = new Gadget(publishGadgetURL );
		button.setProperty("label", "Store in workspace");
		button.setProperty("id", "primer");
		button.setProperty("primer1", SequenceUtils.Serialize(primer1));
		button.setProperty("primer2", SequenceUtils.Serialize(primer2));
		output.append(button);
		output.append("\n Note: Once you have stored the primers in the workspace they both will be in 5' -> 3' direction");
	}

	public void onButtonClicked(Event event, MenuItem menuItem,
			 AbstractSynBioWaveRobot servlet) {
		event.getBlip().reply().append("Primer Designer HowTo: \n\n" +
				"1. Add the sequence you want to amplify to the workspace and select it as active\n"+
				"2. Select the RFC you want to use for your primer\n" +
				"3. Specify the desired melting temperature\n" +
				"4. Click Send\n" +
				"\n The Robot will then generate the primers and display the corresponding Tlow and Thigh");
	}
	
	private RichSequence GeneratePrimer(RichSequence target, double goalTemp, int offset) throws Exception {
		
		RichSequence primer = new RichSequence();
		int i = 0;
		char lastchar = 'x';
		StringBuilder output = new StringBuilder("");
		
		while (i < MIN_PRIMERLENGTH || 
				RichSequence.getMeltingPoint(output.toString()) < goalTemp || 
				(lastchar != 'G' && lastchar != 'C'))
		{
			if(i >= target.getSequence().length()) throw new Exception();
			char nextchar = target.getSequence().charAt(i + offset);
			output.append(nextchar);
			lastchar = nextchar;
			i++;
		}
		primer.setSequence(output.toString());
		return primer;
	}
	
	private void EvaluateForm(Formular form, int sequenceLength) {
		ArrayList<MenuItem> textFields =  form.getPureForm().getSubItemsOfType("textfield");
		try
		{
			temp = Integer.parseInt(textFields.get(0).getValue());
		}catch(NumberFormatException e)
		{
			// use default
			temp = 60;		
		}
		
		// try to read the start and stop position
		try
		{
			start = Integer.parseInt(textFields.get(1).getValue()) - 1;
		}catch(NumberFormatException e)
		{
			start = 0;
		}
		finally
		{
			if(start < 0) start = 0;
		}
		
		String endS = textFields.get(2).getValue();
		if(endS.contentEquals(""))
		{
			end = 0;
		}else
		{
			try
			{
				end = sequenceLength - Integer.parseInt(endS);
			}catch(NumberFormatException e)
			{
				end = 0;
			}
			finally 
			{
				if(end < 0) end = 0;
				if(end > sequenceLength) end = sequenceLength;
			}
		}
	}
}
